#===============================================================================
# Name   : Script do seminrio: Debug no R
# Author : Maria E. G. Alves
#          Augusto F. M. Barros
# Date   : 2023.II
# Aim    : Apresentar os recursos bsicos do R para debug
# Mail   : <<<joseclaudio.faria@gmail.br>>>
#===============================================================================
# Improved:
# - 2024-01-29 19:46:22 - by J.C.Faria
#===============================================================================

#. Introduo:  erros, avisos, mensagens
#.. Erro
erro_fatal <- function(x) {
  if (x < 0) {
     stop("Erro Fatal: O argumento no pode ser menor que
           zero!")
  } else {
    return(sqrt(x))
  }
}

# Testando a funo
erro_fatal(4)
erro_fatal(-2) # Gera erro fatal

#.. Aviso
aviso_exemplo <- function(vec) {
  if (any(vec < 0)) {
     warning("Alguns elementos do vetor so negativos.
              Isso pode causar problema!")
  } else {
    return(sqrt(vec))
  }
}

# Testando a funo
aviso_exemplo(c(4, 1, 9))
aviso_exemplo(c(4, -1, 9)) # Gera um aviso

#.. Mensagem
mensagem_exemplo <- function(x=5) {
  if (missing(x))
    message("Voc no forneceu um valor para 'x'. ",
            "Ser usado o valor padro: ", x)
   return(sqrt(x))
}

# Testando a funo
mensagem_exemplo(9)
mensagem_exemplo() # Gera uma mensagem
(resultado <- suppressMessages(mensagem_exemplo())) # Suprime a mensagem de erro

#.. print()
funcao_com_print <- function(x, y) {
  z <- x + y

  print(paste('O valor de x :', x))
  print(paste('O valor de y :', y))
  print(paste('O valor de z :', z))
}

# Testando a funo
funcao_com_print(3, 5)

#.. cat()
funcao_com_cat <- function(x, y) {
  z <- x + y

  cat('O valor de x :', x,
      '\nO valor de y :', y,
      '\nO valor de z :', z,
      '\n')

}

# Testando a funo
funcao_com_print(3, 5)


#. Ajuda
help(sd)
? sd
help.search('sd')
?? sd


#. traceback()
f <- function(x) {
  r <- x-g(x)
  r
}

g <- function(y) {
  r <- y*h(y)
  r
}

h <- function(z) {
  r <- log(z)

  if(r < 10)
    r^2
  else
    r^3
}

f(-1)
traceback()


#. browser()
(x <- list(2:6, 'a'))

# Exemplo_1
fun <- function(x) {
  browser() # Cria ponto de interrupo e observao
  for (i in seq_along(x))
    z[[i]] <-  x[[i]]+1

  return(z)
}
fun(x)

# Exemplo_2
h <- function(z) {
  browser() # Cria ponto de interrupo e observao
  r <- log(z)
  if(r < 10)
    r^2
  else
    r^3
}

f(-1)


#. debug() e undebug
h <- function(z) {
  r <- log(z)
  if(r < 10)
    r^2
  else
    r^3
}

debug(f)
f(-1)
f(-1) # A funo 'f' continua no modo debug

undebug(f)
f(-1)

debugonce(f)
f(-1)
f(-1) # No entra no modo de debug


#. trace() e untrace()
# Exemplo_1
h <- function(z) {
  r <- log(z)
  if(r < 10)
    r^2
  else
    r^3
}

trace("h",
      tracer=quote(browser()),
      print=T)  
f(-1)
f(-1) # A funo 'h' continua sendo rastreada

untrace("h")
f(-1)

# Exemplo_2
fun <- function(a) {
    x <- a*2
    y <- a + 1
    j <- 1       # linha 4
    k <- 5
    z <- x+y
    u <- 2
    u-1          # linha 8
}

# Depurao forada nas linhas 4 e 8
trace("fun", 
      tracer=quote(browser()), 
      at=c(4,8), 
      print=T) 
fun(3)        
untrace('fun')
fun(3)

#.. Usando condio
trace("fun", 
      tracer=quote(if(y==3){
         browser()
      }), 
      at=4, 
      print=T)
      
fun(2)
fun(3) # No atende a condio

# Tirando pontos de depurao
untrace("fun")
fun(2)
fun(3)

# Exemplo_3
fun_2 <- function(a) {
    x <- a*2
    y <- a+1
    u <- a+5
    j <- 1       # linha 5
    k <- 5
    y <- 1
    z <- x+y
    u <- 2
    u-1
}

trace("fun_2", 
      tracer=quote(if(y==3 | u==2){
         browser()
      }), 
      at=5, 
      print=T)
fun_2(2)  # Atende codio y==3
fun_2(-3) # Atende codio u==2

untrace("fun_2")

# Repetindo funo 'h'
h <- function(z) {
  r <- log(z)
  if(r < 10)
    r^2
  else
    r^3
}

# Usando condio
trace("h",
      quote(if(is.nan(r)) {
         browser()
      }), 
      at=3, 
      print=F)
f(1)
f(-1)
untrace("h")


#. recover()
f <- function(x) {
  a <- 7
  b <- a+1
  r <- (x-g(x)+b); r
}

g <- function(y) {
  a <- 2
  r <- y*h(y); r
  b <- a+1
}

h <- function(z) {
  a <- 24
  b <- a+1
  r <- log(z)
  c <- 3
  
  if(r < 10)
    r^2
  else
    r^3
}

#. --->>> Erro no tinn-r
options(error = recover)
f(-1)
options(error = NULL)
f(-1)


#. try()
# Exemplo_1
(x <- list(1,
          "a",
          2))

z <- list()
for (i in seq_along(x)) {
    z[[i]] <- try(x[[i]] + 1)
}
z

# Suprimindo mensagem de erro com argumento 'silent'
z <- list()
for (i in seq_along(x)) {
    z[[i]] <- try(x[[i]] + 1,
                  silent=T)
}
z

# Exemplo_2
# Passando blocos maiores para try()
teste <- try(
  {
    a <- 1
    b <- "x"
    a + b
  }
)

teste <- try(
  {
    a <- 1
    b <- "x"
    a + b
  }
, silent=T)
teste

# Exemplo_3
# Capturando a sada de try()
(sucesso <- try(1 + 2))
class(sucesso)

sem_sucesso <- try("a" + "b") # Produz erro
class(sem_sucesso)


#. tryCatch()
# Exemplo_1
show_condition <- function(code) {
  cap <- 'capturado por tryCatch'

  tryCatch(code,
    error   = function(c) paste("error:",
                                 cap),

    warning = function(c) paste("warning:",
                                 cap),

    message = function(c) paste("message:",
                                 cap)
  )
}

stop("!") # Erro: !
show_condition(stop("!"))
#> [1] "error"

warning("?!") # Warning message: ?!
show_condition(warning("?!"))
#> [1] "warning"

message("?") # ?
show_condition(message("?"))
#> [1] "message"

# If no condition is captured, tryCatch returns
# the value of the argument
show_condition(10)
# [1] 10

# Exemplo_2
log_and_divide <- function(x, y){
    tryCatch(
        expr={
          result = log10(x) / y
          return(result)
        },
        error=function(e) {
          message('Um ERRO ocorreu!')
          print(e)
        },
        warning=function(w) {
            message('Um WARNING ocorreu!')
            print(w)
            return(NaN)
        }
    )
}

log_and_divide(10, 2)  # Normal
log_and_divide(10)     # Erro
log_and_divide(-10, 2) # Warning

# Exemplo_3
# Funo que pode gerar um erro usando 'finally'
funcao_com_erro <- function(x) {
  tryCatch(
    expr={
      if (x < 0)
         stop("Erro: o argumento deve ser no-negativo!")
         
      res <- sqrt(x)

      cat("Resultado:",
          res,
          "\n")

      return(res)
    }, 

    error = function(e) {
      cat("Erro capturado:",
          conditionMessage(e),
          "\n")
    },   

    finally={
      cat("Limpeza: Esta mensagem ser exibida independentemente do    
           resultado ou erro.\n")
    }
  )
}

funcao_com_erro(9)
funcao_com_erro()
funcao_com_erro(-7)